/*
** cmd.c for ftpclient in /home/chary_c/Prog/my_ftp/client
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Fri Jan  4 13:18:14 2008 corentin chary
** Last update Sun Apr 13 19:18:57 2008 corentin chary
*/

#include <string.h>
#include <stdio.h>
#include <limits.h>

#include "client.h"
#include "cmds.h"

#define CMD_ARG_INF	(UINT_MAX)

static int	cmd_help(t_cmd *cmd);

static const t_bi_cmd builtins[] =
  {
    {
      "help", cmd_help, 0, 1,
      "Show this help.",
      "help [cmd]"
    },
    {
      "quit", cmd_quit, 0, 0,
      "Just quit.",
      "quit"
    },
    {
      "cd", cmd_cd, 0, 1,
      "Change distant directory.",
      "cd [path]"
    },
    {
      "nls", cmd_nls, 0, 1,
      "List distant directory.",
      "nls [path]"
    },
    {
      "ls", cmd_ls, 0, 2,
      "List distant directory.",
      "ls [-r] [path]"
    },
    {
      "pwd", cmd_pwd, 0, 0,
      "Print distant working directory.",
      "pwd"
    },
    {
      "get", cmd_get, 1, CMD_ARG_INF,
      "Download file(s).",
      "get file1 [file2 [file3 [...]]] "
    },
    {
      "put", cmd_put, 1, CMD_ARG_INF,
      "Upload file(s).",
      "put file1 [file2 [file3 [...]]] "
    },
    {
      "rename", cmd_rename, 2, 2,
      "Rename file.",
      "rename src dst"
    },
    {
      "delete", cmd_delete, 1, 1,
      "Delete file.",
      "delete file"
    },
    {
      "mkdir", cmd_mkdir, 1, 1,
      "Create directory.",
      "mkdir dir"
    },
    {
      "rmdir", cmd_rmdir, 1, 1,
      "Remove directory.",
      "rmdir dir"
    },
    {
      "chmod", cmd_chmod, 2, 2,
      "Change mode.",
      "chmod mode file"
    },
    {
      "lcd", cmd_lcd, 1, 1,
      "Change local directory.",
      "lcd path"
    },
    {
      "lls", cmd_lls, 0, 1,
      "List local directory.",
      "lls [path]"
    },
    {
      "lpwd", cmd_lpwd, 0, 0,
      "Print local working directory.",
      "lpwd"
    },
    {NULL, NULL, 0, 0, NULL, NULL}
  };

static const t_bi_cmd	*bi_find(const char *name)
{
  const t_bi_cmd	*bi;

  bi = builtins;
  while (bi->bi_func && strcmp(bi->bi_name, name))
    bi++;
  return (bi->bi_func ? bi : NULL);
}

static int		cmd_help(t_cmd *cmd)
{
  const t_bi_cmd	*bi;

  if (cmd->cm_argc == 2)
    {
      bi = bi_find(cmd->cm_argv[1]);
      if (bi)
	printf("%s:\n%s\n", bi->bi_usage, bi->bi_description);
      else
	fprintf(stderr, "Unknown command %s\n", cmd->cm_argv[1]);
    }
  else
    {
      bi = builtins;
      while (bi->bi_func)
	{
	  printf("%s - %s\n", bi->bi_name, bi->bi_description);
	  bi++;
	}
    }
  return (0);
}

int			execcmd(t_cmd *cmd)
{
  const t_bi_cmd	*bi;

  if (cmd->cm_argc)
    bi = bi_find(cmd->cm_argv[0]);
  else
    bi = NULL;
  if (bi)
    {
      if (cmd->cm_argc - 1 >= bi->bi_min_args
	  && cmd->cm_argc - 1 <= bi->bi_max_args)
        return (bi->bi_func(cmd));
      else
	fprintf(stderr, "%s:\n%s\n", bi->bi_usage, bi->bi_description);
    }
  else
    fprintf(stderr, "Unknown command %s\n", cmd->cm_argv[0]);
  return (1);
}
