/*
** cmd_get.c for ftpclient in /home/chary_c/Prog/my_ftp/client
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Fri Jan  4 16:53:39 2008 corentin chary
** Last update Sun Apr 13 19:43:03 2008 corentin chary
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include "client.h"
#include "cmds.h"

static int	create_file(const char *file, size_t *offset)
{
  struct stat	st;
  int		flags;

  flags = O_WRONLY;
  *offset = 0;
  if (!stat(file, &st))
    {
      printf("File exists, do you want to resume download ? [Y/n]");
      if (getchar() == 'n')
	flags |= O_CREAT | O_TRUNC;
      else
	*offset = st.st_size;
    }
  else
    flags |= O_CREAT;
  return (open(file, flags, 0664));
}

static int	loop(t_ftp *ftp, size_t total, size_t offset)
{
  time_t	st_time;
  size_t	done;
  int		ret;

  done = offset;
  st_time = time(NULL);
  while ((ret = ftp_recv(ftp)) > 0)
    {
      done += ret;
      print_eta(time(NULL) - st_time, done, total);
    }
  putchar('\n');
  return (ret > 0 ? 0 : ret);
}

static int	download_file(t_ftp *ftp, const char *file)
{
  size_t	offset;
  ssize_t	total;
  int		fd;

  fd = create_file(file, &offset);
  if (fd < 0)
    return (-1);
  total = ftp_size(ftp, file);
  if (total < 0)
    goto fail;
  if (ftp_get(ftp, file, offset, fd))
    goto fail;
  if (loop(ftp, total, offset))
    goto fail;
  close(fd);
  return (0);
 fail:
  close(fd);
  return (-1);
}

int	cmd_get(t_cmd *cmd)
{
  for (uint i = 1; i < cmd->cm_argc; ++i)
    if (download_file(cmd->cm_ftp, cmd->cm_argv[i]))
      return (-1);
  return (0);
}
