/*
** cmd_get.c for ftpclient in /home/chary_c/Prog/my_ftp/client
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Fri Jan  4 16:53:39 2008 corentin chary
** Last update Fri Jan  4 21:11:18 2008 corentin chary
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include "client.h"
#include "cmds.h"

static int	open_file(const char *file, size_t *size)
{
  struct stat	st;

  *size = 0;
  if (!stat(file, &st))
    *size = st.st_size;
  return (open(file, O_RDONLY));
}

static int	loop(t_ftp *ftp, size_t total, size_t offset)
{
  time_t	st_time;
  size_t	done;
  int		ret;

  done = offset;
  st_time = time(NULL);
  while ((ret = ftp_send(ftp)) > 0)
    {
      done += ret;
      print_eta(time(NULL) - st_time, done, total);
    }
  putchar('\n');
  return (ret > 0 ? 0 : ret);
}

static ssize_t	get_offset(t_ftp *ftp, const char *file)
{
  ssize_t	offset;

  offset = ftp_size(ftp, file);
  if (offset < 0)
    offset = 0;
  if (offset)
    {
      printf("File exists, do you want to resume upload ? [Y/n]");
      if (getchar() == 'n')
	offset = 0;
    }
  return (offset);
}

static int	upload_file(t_ftp *ftp, const char *file)
{
  ssize_t	offset;
  size_t	total;
  int		fd;

  fd = open_file(file, &total);
  if (fd < 0)
    return (-1);
  offset = get_offset(ftp, file);
  if (ftp_put(ftp, file, offset, fd))
    goto fail;
  if (loop(ftp, total, offset))
    goto fail;
  close(fd);
  return (0);
 fail:
  close(fd);
  return (-1);
}

int	cmd_put(t_cmd *cmd)
{
  for (uint i = 1; i < cmd->cm_argc; ++i)
    if (upload_file(cmd->cm_ftp, cmd->cm_argv[i]))
      return (-1);
  return (0);
}
