/*
** ftplib.h for ftplib in /home/chary_c/Prog/my_ftp/libftp
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Sun Dec 30 13:37:06 2007 corentin chary
** Last update Sun Apr 13 19:23:19 2008 corentin chary
*/

#ifndef FTPLIB_H_
# define FTPLIB_H_

# include <unistd.h>
# include <sys/types.h>
# include <sys/socket.h>
# include <netdb.h>
# include <netinet/in.h>
# include <resolv.h>

# define FTP_TIMEOUT	30000
# define FTP_PORT	21
# define FTP_BUFSIZE	4096

# define FTP_TYPE_PASV	1
# define FTP_TYPE_ACTV	2

# define FTP_IPBUF	(sizeof("255,255,255,255,255,255"))

# define FTP_ASSERT(cond)			\
  do						\
    {						\
      if (!(cond))				\
	return (-1);				\
    }						\
  while (0)

typedef struct	s_ftp_data
{
  int		type;	/* PASV / ACTV */
  int		fd;	/* Data fd */
  int		listen; /* Listener fd */
  int		local;	/* Local fd */
}		t_ftp_data;

typedef struct		s_ftp
{
  /* Global */
  char			*host;
  unsigned short	port;
  int			timeout;
  struct sockaddr	addr;
  /* Command */
  int			cmdfd;
  /* Data */
  t_ftp_data		*data;
  /* Buf */
  char			inbuf[FTP_BUFSIZE];
  size_t		inbuf_len;
  /* Resp */
  int			resp;
  char			resptxt[FTP_BUFSIZE];
}			t_ftp;

t_ftp	*ftp_connect(const char *host, short port, int timeout);
int	ftp_quit(t_ftp *ftp);
int	ftp_close(t_ftp *ftp);

int	ftp_login(t_ftp *ftp, const char *user, const char *pass);

int	ftp_syst(t_ftp *ftp, char *buf, size_t buflen);

int	ftp_opendata(t_ftp *ftp);
int	ftp_closedata(t_ftp *ftp);

int	ftp_pasv(t_ftp *ftp, int pasv);
int	ftp_port(t_ftp *ftp);

int	ftp_cmd(t_ftp *ftp, const char *fmt, ...);
int	ftp_resp(t_ftp *ftp);

int	ftp_pwd(t_ftp *ftp, char *buf, size_t buflen);
int	ftp_chdir(t_ftp *ftp, const char *dir);

int	ftp_mkdir(t_ftp *ftp, const char *dir, char *buf, size_t buflen);
int	ftp_rmdir(t_ftp *ftp, const char *dir);

int	ftp_chmod(t_ftp *ftp, const char *filename, int mode);
int	ftp_alloc(t_ftp *ftp, const int size);

char**	ftp_nlist(t_ftp *ftp, const char *path);
char**	ftp_list(t_ftp *ftp, const char *path, int recursive);

int	ftp_get(t_ftp *ftp, const char *path, int offset, int fd);
int	ftp_put(t_ftp *ftp, const char *path, int offset, int fd);
ssize_t	ftp_send(t_ftp *ftp);
ssize_t	ftp_recv(t_ftp *ftp);

ssize_t	ftp_size(t_ftp *ftp, const char *path);

int	ftp_rename(t_ftp *ftp, const char *src, const char *dst);
int	ftp_delete(t_ftp *ftp, const char *path);

const char	*ftp_error(t_ftp *ftp);
void	ftp_mkip(struct in_addr addr, short port, char ipbuf[FTP_IPBUF]);

#endif /* !FTPLIB_H_ */
