/*
** cmd.c for ftpclient in /home/chary_c/Prog/my_ftp/client
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Fri Jan  4 13:18:14 2008 corentin chary
** Last update Sun Apr 13 19:13:34 2008 corentin chary
*/

#include <string.h>
#include <stdio.h>
#include <limits.h>

#include "server.h"
#include "cmds.h"

#define CMD_ARG_INF	(UINT_MAX)

static const t_bi_cmd builtins[] =
  {
    {"QUIT", cmd_quit, 0, 0, 0},
    {"USER", cmd_user, 1, 1, 0},
    {"PASS", cmd_pass, 1, 1, 0},
    {"NOOP", cmd_noop, 0, 0, 0},
    {"PWD", cmd_pwd, 0, 0, 0},
    {"CWD", cmd_cwd, 1, 1, 0},
    {"RNFR", cmd_rnfr, 1, 1, 0},
    {"RNTO", cmd_rnto, 1, 1, 0},
    {"SYST", cmd_syst, 0, 0, 0},
    {"TYPE", cmd_type, 1, 1, 0},
    {"STRU", cmd_stru, 1, 1, 0},
    {"MODE", cmd_mode, 1, 1, 0},
    {"PORT", cmd_port, 1, 1, 0},
    {"MKD", cmd_mkd, 1, 1, 0},
    {"RMD", cmd_rmd, 1, 1, 0},
    {"DELE", cmd_dele, 1, 1, 0},
    {"SIZE", cmd_size, 1, 1, 0},
    {"REST", cmd_rest, 1, 1, 0},
    {"NLST", cmd_nlst, 0, 1, 1},
    {"LIST", cmd_list, 0, 1, 1},
    {"RETR", cmd_retr, 1, 1, 1},
    {"STOR", cmd_stor, 1, 1, 1},
    {NULL, NULL, 0, 0, 0}
  };

static const t_bi_cmd	*bi_find(const char *name)
{
  const t_bi_cmd	*bi;

  bi = builtins;
  while (bi->bi_func && strcasecmp(bi->bi_name, name))
    bi++;
  return (bi->bi_func ? bi : NULL);
}

static int	handle_state(t_cmd *cmd, const t_bi_cmd *bi)
{
  int		(*func)(t_cmd *cmd);
  int		state;
  t_ftp		*ftp;

  ftp = cmd->cm_ftp;
  state = ftp->state;
  func = bi->bi_func;
  if (func == cmd_quit)
    return (0);
  if (state == FTP_STATE_LOGIN && (func != cmd_pass && func != cmd_user))
    {
      resp(ftp, 501, "Please login with USER and PASS.");
      return (-1);
    }
  if (bi->bi_data && !ftp->data)
    {
      resp(ftp, 425, "Use PORT first.");
      return (-1);
    }
  return (0);
}

int			execcmd(t_cmd *cmd)
{
  const t_bi_cmd	*bi;

  if (cmd->cm_argc)
    bi = bi_find(cmd->cm_argv[0]);
  else
    bi = NULL;
  if (bi)
    {
      if (handle_state(cmd, bi))
	return (-1);
      if (cmd->cm_argc - 1 >= bi->bi_args_min &&
	  cmd->cm_argc - 1 <= bi->bi_args_max)
        return (bi->bi_func(cmd));
      else
	resp(cmd->cm_ftp, 501, "Syntax error in parameters or arguments.");
    }
  else
    resp(cmd->cm_ftp, 500, "Syntax error, command unrecognized...");
  return (-1);
}
