/*
** cmd_nlst.c for ftpserver in /home/chary_c/Prog/my_ftp/server
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Tue Jan  8 08:37:01 2008 corentin chary
** Last update Sun Apr 13 20:18:25 2008 corentin chary
*/

#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <time.h>

#include "sock.h"
#include "error.h"
#include "server.h"
#include "cmds.h"

#define MO(st, r, ct)				\
  ((st)->st_mode & (r) ? (ct) : '-')

static char	get_type(mode_t m)
{
  if (S_ISREG(m))
    return ('-');
  if (S_ISDIR(m))
    return ('d');
  if (S_ISCHR(m))
    return ('c');
  if (S_ISBLK(m))
    return ('b');
  if (S_ISFIFO(m))
    return ('f');
  if (S_ISLNK(m))
    return ('l');
  if (S_ISSOCK(m))
    return ('s');
  return ('?');
}

static void	put_str(char *buf, size_t buflen,
			struct stat *st, const char *name)
{
  char		lbuf[64];
  struct tm	tm;

  buf[0] = '\0';
  snprintf(lbuf, sizeof (lbuf), "%c%c%c%c%c%c%c%c%c%c %5d ",
	   get_type(st->st_mode),
	   MO(st, S_IRUSR, 'r'), MO(st, S_IWUSR, 'w'), MO(st, S_IXUSR, 'x'),
	   MO(st, S_IRGRP, 'r'), MO(st, S_IWGRP, 'w'), MO(st, S_IXGRP, 'x'),
	   MO(st, S_IROTH, 'r'), MO(st, S_IWOTH, 'w'), MO(st, S_IXOTH, 'x'),
	   (int) st->st_nlink);
  strlcat(buf, lbuf, buflen);
  snprintf(lbuf, sizeof (lbuf), "%5d     %5d ", st->st_uid, st->st_gid);
  strlcat(buf, lbuf, buflen);
  snprintf(lbuf, sizeof (lbuf), "%10d ", (int) st->st_size);
  strlcat(buf, lbuf, buflen);
  localtime_r(&st->st_mtime, &tm);
  strftime(lbuf, sizeof (lbuf), "%b %C %H:%M ", &tm);
  strlcat(buf, lbuf, buflen);
  strlcat(buf, name, buflen);
  strlcat(buf, "\r\n", buflen);
}

static int	loop(t_ftp *ftp, DIR *dip, const char *dir)
{
  char		buf[FTP_BUFSIZE];
  char		path[PATH_MAX];
  struct dirent	*dit;
  struct stat	st;

  while ((dit = readdir(dip)) != NULL)
    {
      if (!strcmp(dit->d_name, ".") || !strcmp(dit->d_name, ".."))
	continue ;
      strlcpy(path, dir, sizeof (path));
      strlcat(path, "/", sizeof (path));
      strlcat(path, dit->d_name, sizeof (path));
      if (stat(path, &st))
	continue ;
      put_str(buf, sizeof (buf), &st, dit->d_name);
      if (sock_send_all(ftp->data->fd, buf, strlen(buf), ftp->timeout) < 0)
	return (-1);
    }
  sock_send_all(ftp->data->fd, "\r\n", 2, ftp->timeout);
  return (0);
}

int		cmd_list(t_cmd *cmd)
{
  t_ftp		*ftp;
  char		*dir;
  DIR		*dip;
  int		ret;

  ret = 0;
  ftp = cmd->cm_ftp;
  dip = NULL;
  dir = (cmd->cm_argc == 2  && *cmd->cm_argv[1] != '-' ? cmd->cm_argv[1] : ".");
  ret = resp(ftp, 150, "Here comes the directory listing.");
  if (check_dir(ftp, dir) || (dip = opendir(dir)) == NULL)
    {
      resp(ftp, 226, "Transfer done (but failed to open directory)");
      perr(errno, "opendir(%s)", dir);
      ret = -1;
    }
  else
    {
      ret = loop(ftp, dip, dir);
      resp(ftp, 226, "Directory send OK.");
    }
  close_data(ftp);
  closedir(dip);
  return (ret);
}
