/*
** cmd_retr.c for ftpserver in /home/chary_c/Prog/my_ftp/server
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Mon Jan  7 21:17:31 2008 corentin chary
** Last update Sun Apr 13 20:09:40 2008 corentin chary
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "error.h"
#include "sock.h"
#include "server.h"
#include "cmds.h"

static int	open_file(t_ftp *ftp, const char *file, size_t *size)
{
  struct stat	st;
  int		fd;
  size_t	rest;

  rest = ftp->rest;
  ftp->rest = 0;
  *size = 0;
  if (check_file(ftp, (char *)file))
    return (-1);
  if (!stat(file, &st))
    *size = st.st_size;
  fd = open(file, O_RDONLY);
  if (fd >=0 && lseek(fd, rest, SEEK_SET) < 0)
    {
      perr(errno, "lseek()");
      fd = -1;
    }
  return (fd);
}

static int	loop(int fd, t_ftp *ftp)
{
  int		ret;
  char		buf[FTP_BUFSIZE];
  int		sockfd;

  sockfd = ftp->data->fd;
  while ((ret = read(fd, buf, sizeof (buf))) > 0)
    {
      ret = sock_send_all(sockfd, buf, ret, ftp->timeout);
      if (ret < 0)
	break ;
    }
  return (ret >= 0 ? 0 : -1);
}

static int	upload_file(t_ftp *ftp, const char *file)
{
  size_t	total;
  int		fd;

  fd = open_file(ftp, file, &total);
  if (fd < 0)
    goto fail;
  resp(ftp, 150, "Opening BINARY mode data connection for %s (%u bytes).",
       file, total);
  if (loop(fd, ftp))
    goto fail;
  resp(ftp, 226, "File send OK.");
  close(fd);
  return (0);
 fail:
  resp(ftp, 550, "Transfert failed, file probably not found.");
  if (fd >= 0)
    close(fd);
  return (-1);
}

int	cmd_retr(t_cmd *cmd)
{
  int	ret;
  char	*file;
  t_ftp	*ftp;

  ftp = cmd->cm_ftp;
  file = cmd->cm_argv[1];
  ret = upload_file(ftp, file);
  close_data(ftp);
  return (ret);
}
