/*
** cmd_stor.c for ftpserver in /home/chary_c/Prog/my_ftp/server
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Mon Jan  7 21:17:31 2008 corentin chary
** Last update Sun Apr 13 20:09:52 2008 corentin chary
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "error.h"
#include "sock.h"
#include "server.h"
#include "cmds.h"

static int	open_file(t_ftp *ftp, const char *file)
{
  int		fd;
  int		flags;
  size_t	rest;

  rest = ftp->rest;
  ftp->rest = 0;
  if (check_file(ftp, (char *)file))
    return (-1);
  flags = O_WRONLY;
  if (!rest)
    flags |= O_CREAT | O_TRUNC;
  if ((fd = open(file,  flags, 0644)) < 0)
    perr(errno, "open(%s)", file);
  if (fd >=0 && lseek(fd, rest, SEEK_SET) < 0)
    {
      perr(errno, "lseek()");
      fd = -1;
    }
  return (fd);
}

static int	loop(int fd, t_ftp *ftp)
{
  int		ret;
  char		buf[FTP_BUFSIZE];
  int		sockfd;

  sockfd = ftp->data->fd;
  while ((ret = sock_recv(sockfd, buf, sizeof (buf), ftp->timeout)) > 0)
    {
      ret = write(fd, buf, ret);
      if (ret < 0)
	break ;
    }
  return (ret >= 0 ? 0 : -1);
}

static int	download_file(t_ftp *ftp, const char *file)
{
  int		fd;

  fd = open_file(ftp, file);
  if (fd < 0)
    goto fail;
  resp(ftp, 150, "Ok to send data.");
  if (loop(fd, ftp))
    goto fail;
  resp(ftp, 226, "File receive OK.");
  close(fd);
  return (0);
 fail:
  resp(ftp, 550, "Transfert failed.");
  if (fd >= 0)
    close(fd);
  return (-1);
}

int	cmd_stor(t_cmd *cmd)
{
  int	ret;
  char	*file;
  t_ftp	*ftp;

  ftp = cmd->cm_ftp;
  file = cmd->cm_argv[1];
  ret = download_file(ftp, file);
  close_data(ftp);
  return (ret);
}
