/*
** logging.c for ftpserver in /home/chary_c/Prog/my_ftp/server
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Sun Apr 13 18:02:11 2008 corentin chary
** Last update Sun Apr 13 19:39:18 2008 corentin chary
*/

#ifdef __linux__
# define USE_SHADOW
#endif

#ifdef USE_SHADOW
# include <shadow.h>
# include <crypt.h>
#endif
#include <unistd.h>
#include <pwd.h>
#include <string.h>
#include <time.h>
#include <stdio.h>
#include <errno.h>

#include "error.h"
#include "strlcpy.h"
#include "server.h"

static int	check_shell(const struct passwd *pwd)
{
  const char	*shell;

  while ((shell = getusershell()) && strcmp(shell, pwd->pw_shell));
  endusershell();
  if (shell == NULL)
    return (-1);
  return (0);
}

#ifdef USE_SHADOW
static int	check_shadow(char *user, char *pass)
{
  const char		*pass_crypt;
  const struct spwd	*spwd;
  int			days;

  spwd = getspnam(user);
  if (!spwd)
    return (-1);
  days = time(NULL) / (60 * 60 * 24);
  if (spwd->sp_expire > 0 && spwd->sp_expire < days)
    return (-1);
  if (spwd->sp_lstchg > 0 && spwd->sp_max > 0 &&
      spwd->sp_lstchg + spwd->sp_max < days)
    return (-1);
  pass_crypt = crypt(pass, spwd->sp_pwdp);
  return (strcmp(pass_crypt, spwd->sp_pwdp));
}
#else
static int	check_shadow(char *user, char *pass)
{
  (void) user;
  (void) pass;
  return (-1);
}
#endif

static int	do_chroot(const struct passwd *pwd, char *dir)
{
  if (chdir(dir))
    {
      perr(errno, "chdir(%s)", dir);
      return (-1);
    }
  if (!getuid() && chroot(dir))
    {
      perr(errno, "chroot(%s)", dir);
      return (-1);
    }
  if (!getuid() && pwd)
    {
      if (setgid(pwd->pw_gid) || setuid(pwd->pw_uid))
	{
	  perr(errno, "setuid(%d)", pwd->pw_uid);
	  return (-1);
	}
    }
  return (0);
}

int			login(t_ftp *ftp, char *password)
{
  const char		*pass_crypt;
  const struct passwd	*pwd;

  if (!strcmp(ftp->user, "anonymous"))
    {
      ftp->is_ano = 1;
      getcwd(ftp->home, sizeof (ftp->home));
      return (do_chroot(getpwnam("nobody"), ftp->home));
    }
  pwd = getpwnam(ftp->user);
  if (!pwd)
    return (-1);
  if (check_shell(pwd))
    return (-1);
  if (check_shadow(ftp->user, password))
    {
      pass_crypt = crypt(password, pwd->pw_passwd);
      if (strcmp(pass_crypt, pwd->pw_passwd))
	return (-1);
    }
  strlcpy(ftp->home, pwd->pw_dir, sizeof (ftp->home));
  return (do_chroot(pwd, ftp->home));
}
